
import java.lang.reflect.*;
import java.util.*;
import java.io.*;
import parser.BulletNotML;
import parser.BulletNotML.*;

public class Main {
    public static void main(String[] args) {
        try {
            Bulletml root;
            if (args.length == 0) {
                Reader reader = new InputStreamReader(System.in);
                root = new BulletNotML().parseBulletml("-", reader, 8);
                reader.close();
            } else {
                root = new BulletNotML().parseBulletml(new java.io.File(args[0]), null);
            }
            PrintWriter writer = new PrintWriter(new OutputStreamWriter(System.out));
            writer.println("<?xml version=\"1.0\" ?>");
            writer.println("<!DOCTYPE bulletml SYSTEM \"http://www.asahi-net.or.jp/~cs8k-cyu/bulletml/bulletml.dtd\">");
            output(writer, 0, root);
            writer.close();
            
            //System.out.println(root);
        } catch(Exception x) {
            x.printStackTrace();
        }
    }
    
    private static void output(PrintWriter writer, int indent, Node node) throws Exception {
        
        for (int i = 0; i < indent; i++)
            writer.print("  ");
        
        String name = node.getClass().getName();
        name = name.substring(name.lastIndexOf('.') + 1);
        name = name.substring(name.lastIndexOf('$') + 1);
        int index = name.indexOf('_');
        if (index >= 0)
            name = name.substring(0, index);
        name = uncapitalize(name);
        writer.print("<" + name);
        
        Method[] methods = node.getClass().getMethods();
        for (int i = 0; i < methods.length; i++) {
            Method m = methods[i];
            if (m.getReturnType().equals(Token.class) && m.getParameterTypes().length == 0) {
                Token token = (Token) m.invoke(node, new Object[0]);
                if (token != null) {
                    writer.print(" " + m.getName() + "=\"" + token.getImage() + "\"");
                }
            }
        }
        
        List children = new LinkedList(node.getChildNodes());
        
        Expression expression = null;
        Iterator it = children.iterator();
        while (it.hasNext()) {
            Node child = (Node) it.next();
            if (child instanceof Expression) {
                expression = (Expression) child;
                break;
            }
            if (child instanceof Token) {
                it.remove();
            }
        }
        
        if (children.isEmpty()) {
            writer.println(" />");
            return;
        }
        
        if (expression != null) {
            writer.print(">");
            writer.print(getImage(expression));
            writer.println("</" + name + ">");
            return;
        }
        
        writer.println(">");
        
        it = children.iterator();
        while (it.hasNext()) {
            Node child = (Node) it.next();
            output(writer, indent + 1, child);
        }
        
        for (int j = 0; j < indent; j++)
            writer.print("  ");
        writer.println("</" + name + ">");
        
    }
    
    private static String getImage(Node node) {
        StringBuffer result = new StringBuffer();
        getImage(result, node);
        return result.toString();
    }
    private static void getImage(StringBuffer result, Node node) {
        if (node instanceof Token) {
            Token token = (Token) node;
            if (!token.isWhite()) {
                result.append(token.getImage());
            }
        } else {
            Iterator it = node.getChildNodes().iterator();
            while (it.hasNext()) {
                Node child = (Node) it.next();
                getImage(result, child);
            }
        }
    }
    
    private static String uncapitalize(String str) {
        char ch = str.charAt(0);
        String tail = str.substring(1);
        return Character.toLowerCase(ch) + tail;
    }
    
}

